@@ -701,7 +701,7 @@ static const int8_t unhex[256] =
   const char *body_mark = 0;
   const unsigned int lenient = parser->lenient_http_headers;
   const unsigned int allow_chunked_length = parser->allow_chunked_length;
-  
+
   uint32_t nread = parser->nread;
 
   /* We're in an error state. Don't bother doing anything. */
@@ -1104,7 +1104,7 @@ reexecute:
       }
 
       case s_header_field:
-      {        
+      {
         const char* start = p;
         for (; p != data + len; p++) {
           ch = *p;
@@ -1112,7 +1112,7 @@ reexecute:
 
           if (!c)
             break;
-          
+
           switch (parser->header_state) {
             case h_general: {
               size_t left = data + len - p;
@@ -1273,7 +1273,14 @@ reexecute:
 
         switch (parser->header_state) {
           case h_upgrade:
-            parser->flags |= F_UPGRADE;
+            // Crow does not support HTTP/2 at the moment.
+            // According to the RFC https://datatracker.ietf.org/doc/html/rfc7540#section-3.2
+            // "A server that does not support HTTP/2 can respond to the request as though the Upgrade header field were absent"
+            // => `F_UPGRADE` is not set if the header starts by "h2".
+            // This prevents the parser from skipping the request body.
+            if (ch != 'h' || p+1 == (data + len) || *(p+1) != '2') {
+              parser->flags |= F_UPGRADE;
+            }
             parser->header_state = h_general;
             break;
 
@@ -1285,7 +1292,7 @@ reexecute:
               parser->header_state = h_matching_transfer_encoding_token;
             }
             break;
-            
+
           /* Multi-value `Transfer-Encoding` header */
           case h_matching_transfer_encoding_token_start:
             break;
@@ -1295,7 +1302,7 @@ reexecute:
               CROW_SET_ERRNO(CHPE_INVALID_CONTENT_LENGTH);
               goto error;
             }
-            
+
             if (parser->flags & F_CONTENTLENGTH) {
               CROW_SET_ERRNO(CHPE_UNEXPECTED_CONTENT_LENGTH);
               goto error;
@@ -1352,12 +1359,12 @@ reexecute:
             CROW_CALLBACK_DATA_NOADVANCE(header_value);
             CROW_REEXECUTE();
           }
-          
+
           if (!lenient && !CROW_IS_HEADER_CHAR(ch)) {
             CROW_SET_ERRNO(CHPE_INVALID_HEADER_TOKEN);
             goto error;
           }
-          
+
           c = CROW_LOWER(ch);
 
           switch (h_state) {
@@ -1421,7 +1428,7 @@ reexecute:
               parser->content_length = t;
               break;
             }
-            
+
             case h_content_length_ws:
               if (ch == ' ') break;
               CROW_SET_ERRNO(CHPE_INVALID_CONTENT_LENGTH);
@@ -1499,11 +1506,11 @@ reexecute:
           }
         }
         parser->header_state = h_state;
-        
-        
+
+
         if (p == data + len)
           --p;
-        
+
         CROW_COUNT_HEADER_SIZE(p - start);
         break;
       }
@@ -1579,7 +1586,7 @@ reexecute:
           CROW_CALLBACK_NOTIFY(message_complete);
           break;
         }
-        
+
         /* Cannot use transfer-encoding and a content-length header together
            per the HTTP specification. (RFC 7230 Section 3.3.3) */
         if ((parser->uses_transfer_encoding == 1) &&
@@ -1597,7 +1604,7 @@ reexecute:
             goto error;
           }
         }
-        
+
         parser->state = s_headers_done;
 
         /* Set this here so that on_headers_complete() callbacks can see it */
@@ -1922,9 +1929,8 @@ error:
 inline void
   http_parser_init(http_parser* parser)
 {
-  //void *data = parser->data; /* preserve application data */
   memset(parser, 0, sizeof(*parser));
-  parser->data = NULL;//data;
+  parser->data = NULL;
   parser->state = s_start_req;
   parser->http_errno = CHPE_OK;
 }
